#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Copyright 2019, Nils Hilbricht, Germany ( https://www.hilbricht.net )

This file is part of the Laborejo Software Suite ( https://www.laborejo.org ),
more specifically its template base application.

The Template Base Application is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import template.engine.api #we need direct access to the module to inject data in the provided structures. but we also need the functions directly. next line:
from template.engine.api import *
import template.engine.ly2cbox as ly2cbox


#New callbacks
class ClientCallbacks(Callbacks): #inherits from the templates api callbacks
    def __init__(self):
        super().__init__()
#Inject our derived Callbacks into the parent module
template.engine.api.callbacks = ClientCallbacks()
from template.engine.api import callbacks


_templateStartEngine = startEngine
def startEngine(nsmClient):
    _templateStartEngine(nsmClient)
    
    #Populate Channel Activity and Channel Instrument Changes midi callbacks
    _registerMidiCallbacks()
        
    playTestSignal.patterns = {channel : ly2cbox.pattern("c'16 g c", channel)  for channel in range(1, 17)}    
           
    #Send initial Data etc.        
    callbacks._soundfontChanged()
    callbacks._ignoreProgramChangesChanged()

def _registerMidiCallbacks():
    """This needs to be a function to delay execution until we have a session and nsm loaded
    our data."""
    from template.engine.input_midi import MidiProcessor

    def _NoteOn(channel, m_note, m_velocity):
        callbacks._channelActivity(channel+1)

    def _controlChange(channel, m_note, m_velocity):                
        if m_note == MidiProcessor.CC_BANKCHANGE_COARSE or m_note == MidiProcessor.CC_BANKCHANGE_FINE:                                    
            callbacks._channelChanged(channel+1)
                    
    def _programChange(channel, value):            
        callbacks._channelChanged(channel+1)

    session.data.midiInput.midiProcessor.callbacks3[(MidiProcessor.SIMPLE_EVENT, MidiProcessor.M_NOTE_ON)] = _NoteOn
    session.data.midiInput.midiProcessor.callbacks3[(MidiProcessor.SIMPLE_EVENT, MidiProcessor.M_CONTROL_CHANGE)] = _controlChange
    #session.data.midiInput.midiProcessor.callbacks2[(MidiProcessor.SIMPLE_EVENT, MidiProcessor.PROGRAMCHANGE)] = _programChange


#Sampler
def playTestSignal(channel:int):    
    """Channel 1-16
    Play a short test chime to verify that sound is working and to see how the instrument sounds"""            
    #patterns are defined in startEngine because we obviously need to wait for cbox to be ready.            
    if channel <1 or channel > 16:
        raise ValueError(f"Midi Channel must be between 1 and 16 inclusive. Yours is {channel}")
    callbacks._channelActivity(channel)
    session.data.midiInput.scene.play_pattern(playTestSignal.patterns[channel], 150.0, id=channel) #If there is already an adhoc pattern with a given non-zero id, stop it and release all the pending notes. Retry until all the notes are released.
